@@ -199,7 +199,7 @@ def sqlite_s3_query(url, get_credentials=lambda: (
             c_double_p[0] = time()/86400.0 + 2440587.5;
             return SQLITE_OK
 
-        io_methods = make_struct(tuple((
+        io_methods = make_struct((
             ('i_version', c_int, 1),
             ('x_close', x_close_type, x_close_type(x_close)),
             ('x_read', x_read_type, x_read_type(x_read)),
@@ -213,11 +213,11 @@ def sqlite_s3_query(url, get_credentials=lambda: (
             ('x_file_control', x_file_control_type, x_file_control_type(x_file_control)),
             ('x_sector_size', c_void_p, None),
             ('x_device_characteristics', x_device_characteristics_type, x_device_characteristics_type(x_device_characteristics)),
-        )))
-        file = make_struct(tuple((
+        ))
+        file = make_struct((
             ('p_methods', POINTER(type(io_methods)), pointer(io_methods)),
-        )))
-        vfs = make_struct(tuple((
+        ))
+        vfs = make_struct((
             ('i_version', c_int, 1),
             ('sz_os_file', c_int, sizeof(file)),
             ('mx_pathname', c_int, 1024),
@@ -236,7 +236,7 @@ def sqlite_s3_query(url, get_credentials=lambda: (
             ('x_sleep', c_void_p, None),
             ('x_current_time', x_current_time_type, x_current_time_type(x_current_time)),
             ('x_get_last_error', c_void_p, None),
-        )))
+        ))
 
         run(libsqlite3.sqlite3_vfs_register, byref(vfs), 0)
         try:
@@ -262,28 +262,31 @@ def sqlite_s3_query(url, get_credentials=lambda: (
         finally:
             run_with_db(db, libsqlite3.sqlite3_finalize, pp_stmt)
 
+    @contextmanager
     def query(db, sql, params=()):
         with get_pp_stmt(db, sql) as pp_stmt:
-
             for i, param in enumerate(params):
                 run_with_db(db, bind[type(param)], pp_stmt, i + 1, param)
 
-            row_constructor = namedtuple('Row', (
-                sub('_+', '_', sub(r'\W|^(?=\d)','_', libsqlite3.sqlite3_column_name(pp_stmt, i).decode()).strip('_'))
+            columns = tuple(
+                libsqlite3.sqlite3_column_name(pp_stmt, i).decode()
                 for i in range(0, libsqlite3.sqlite3_column_count(pp_stmt))
-            ))
+            )
 
-            while True:
-                res = libsqlite3.sqlite3_step(pp_stmt)
-                if res == SQLITE_DONE:
-                    break
-                if res != SQLITE_ROW:
-                    raise Exception(libsqlite3.sqlite3_errstr(res).decode())
+            def rows():
+                while True:
+                    res = libsqlite3.sqlite3_step(pp_stmt)
+                    if res == SQLITE_DONE:
+                        break
+                    if res != SQLITE_ROW:
+                        raise Exception(libsqlite3.sqlite3_errstr(res).decode())
 
-                yield row_constructor(*(
-                    extract[libsqlite3.sqlite3_column_type(pp_stmt, i)](pp_stmt, i)
-                    for i in range(0, len(row_constructor._fields))
-                ))
+                    yield tuple(
+                        extract[libsqlite3.sqlite3_column_type(pp_stmt, i)](pp_stmt, i)
+                        for i in range(0, len(columns))
+                    )
+
+            yield columns, rows()
 
     with \
             get_http_client() as http_client, \
